class ErrorCode:
    aborted = 10
    already_exists = 6
    cancelled = 1
    data_loss = 15
    deadline_exceeded = 4
    failed_precondition = 9
    internal = 13
    invalid_argument = 3
    not_found = 5
    ok = 0
    out_of_range = 11
    permission_denied = 7
    resource_exhausted = 8
    unauthenticated = 16
    unavailable = 14
    unimplemented = 12
    unknown = 2


class ErrorMessage:
    def __init__(self, code : int, message : str ):
        self.code = code
        self.message = message
class ErrorException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message


class DBException(ErrorException):
    def __init__(self, code, message):
        self.code = code
        self.message = message

class KeyException(ErrorException):
    def __init__(self, code, message):
        self.code = code
        self.message = message

class AuthException(ErrorException):
    def __init__(self, code, message):
        self.code = code
        self.message = message



from typing import Protocol, List

from algservice.services.algservice.models import Mission, PreprocessResponse, DetectResponse, Algorithm


class AlgorithmService(Protocol):
    def get_version(self) -> str:
        ...

    def get_algorithms(self) -> List[Algorithm]:
        ...

    def pre_process(self, name: str, image: bytes, missions: List[Mission]) -> PreprocessResponse:
        ...

    def detect(self, name: str, image: bytes, missions: List[Mission]) -> DetectResponse:
        ...



import os



from algservice.util.utils import get_user_home, remove_file


def locked( lock = None):

    def real_lock(func):
        def wrapper(self, *args, **kwargs):
            lockobj = getattr(self, lock)
            with lockobj:
                return func(self, *args, **kwargs)

        return wrapper

    return real_lock



def clean_and_save_user_spec_file( filename , content ):
    home = get_user_home()
    fullfile = os.path.join(home, filename)
    remove_file(fullfile);

    with open(fullfile, 'w') as f:
        f.write(str(content))

def read_user_spec_file( filename  ):
    home = get_user_home()
    fullfile = os.path.join(home, filename)
    try:
        with open(fullfile, 'r') as f:
            line = f.readline()
            return line
    except Exception as e:
        print("read file error ", fullfile ,e )
    return None





from datetime import datetime
from typing import Union, List

import grpc


def value_or_empty(s: Union[str, None]) -> str:
    if s is None:
        return ''
    return s


def bytes_or_empty(s: Union[bytes, None]) -> bytes:
    if s is None:
        return b''
    return s


def array_or_empty(s: any) -> list:
    if s is None:
        return []
    return s


def value_or_empty_dict(s: dict, name: str) -> str:
    if name in s:
        return value_or_empty(s[name])
    return ''


def value_or_empty_dict_float(s: dict, name: str) -> int:
    if name in s:
        return value_or_zero(s[name])
    return 0


def value_or_empty_dict_datetime(s: dict, name: str) -> float:
    if name in s:
        return from_datetime_to_float(s[name])
    return 0


def value_or_zero(s: Union[int, None], z: int = 0) -> int:
    if s is None:
        return z
    return s


def value_or_float_zero(s: Union[float, None], z: float = 0.) -> float:
    if s is None:
        return z
    return s


def from_float_to_datetime(s: Union[float, None]) -> Union[datetime, None]:
    if s is None:
        return None
    if s < 0:
        return None
    return datetime.fromtimestamp(s)


def from_datetime_to_float(ts: datetime) -> float:
    if ts is None:
        return -1

    return ts.timestamp()


class RemoveError(Exception):
    """Raised by the gRPC library to indicate non-OK-status RPC termination."""

    def __init__(self, e: grpc.RpcError):
        self.code = e.code()
        self.message = e.details()


def wrapper_rpc_error(func):
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except grpc.RpcError as rpc_error:
            raise RemoveError(rpc_error)

    return wrapper



from typing import List
import grpc
from google.protobuf import empty_pb2

from .alg_service import AlgorithmService
from algservice.services.algservice.models import Mission, DetectResponse, PreprocessResponse, AlgorithmInfos


from .grpcutils import pb2_to_local, local_to_pb2
from .pb2utils import wrapper_rpc_error
from ..grpc.alg_service_pb2 import AlgorithmsRequest_
from ..grpc.alg_service_pb2_grpc import AlgServiceStub

_EMPTY = empty_pb2.Empty()

class GrpcChannel:
    def __init__(self, host='localhost', port=49001):
        addr = f'{host}:{port}'

        self.channel = grpc.insecure_channel(addr)


class AlgServiceClient(AlgorithmService):

    def __init__(self, channel: GrpcChannel):
        self._stub = AlgServiceStub(channel.channel)



    @wrapper_rpc_error
    def get_version(self) -> str :

        response = self._stub.GetVersion(_EMPTY) # AlgorithmsResponse
        return response.version



    @wrapper_rpc_error
    def get_algorithms(self) -> AlgorithmInfos:
        """返回 当前支持的所有算法
        """
        # 对 输入 输出编解码
        req = AlgorithmsRequest_(name="")
        response = self._stub.GetAlgorithms(req) # AlgorithmsResponse
        return pb2_to_local.to_AlgorithmInfos(response)

    @wrapper_rpc_error
    def pre_process(self,   name:str  , image:bytes,   missions: List[Mission]   ) -> PreprocessResponse :
        """预处理，在注册阶段， 背景图+ 图形+算法+参数 需要获取一段兹定于数据
        ： 输入 注册图像，检测任务列表，算法参数， 返回 预处理数据，这个数据应该调用者保存，将来在检测时
        使用
        """

        req = local_to_pb2.to_DetectRequest_(name, image, missions)
        response = self._stub.Preprocess( req )
        return pb2_to_local.to_PreprocessResponse( response )

    @wrapper_rpc_error
    def detect(self,  name:str ,image:bytes,   missions: List[Mission]   ) -> DetectResponse :
        """检测： 输入 实时图像，检测任务列表， 返回 检测结果列表

        """
        req = local_to_pb2.to_DetectRequest_(name, image, missions)
        response = self._stub.Detect(req)
        return pb2_to_local.to_DetectResponse(response)


def create_alg_service_api(host, port) -> AlgServiceClient:
    c = GrpcChannel(host, port)
    return AlgServiceClient(c)



from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ServiceVersion_(_message.Message):
    __slots__ = ("version",)
    VERSION_FIELD_NUMBER: _ClassVar[int]
    version: str
    def __init__(self, version: _Optional[str] = ...) -> None: ...

class Size_(_message.Message):
    __slots__ = ("width", "height")
    WIDTH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    width: float
    height: float
    def __init__(self, width: _Optional[float] = ..., height: _Optional[float] = ...) -> None: ...

class Point_(_message.Message):
    __slots__ = ("x", "y")
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    x: float
    y: float
    def __init__(self, x: _Optional[float] = ..., y: _Optional[float] = ...) -> None: ...

class Rect_(_message.Message):
    __slots__ = ("left", "top", "width", "height")
    LEFT_FIELD_NUMBER: _ClassVar[int]
    TOP_FIELD_NUMBER: _ClassVar[int]
    WIDTH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    left: float
    top: float
    width: float
    height: float
    def __init__(self, left: _Optional[float] = ..., top: _Optional[float] = ..., width: _Optional[float] = ..., height: _Optional[float] = ...) -> None: ...

class Polygon_(_message.Message):
    __slots__ = ("points",)
    POINTS_FIELD_NUMBER: _ClassVar[int]
    points: _containers.RepeatedCompositeFieldContainer[Point_]
    def __init__(self, points: _Optional[_Iterable[_Union[Point_, _Mapping]]] = ...) -> None: ...

class Shape_(_message.Message):
    __slots__ = ("polygon", "box")
    POLYGON_FIELD_NUMBER: _ClassVar[int]
    BOX_FIELD_NUMBER: _ClassVar[int]
    polygon: Polygon_
    box: Rect_
    def __init__(self, polygon: _Optional[_Union[Polygon_, _Mapping]] = ..., box: _Optional[_Union[Rect_, _Mapping]] = ...) -> None: ...

class AlgRequest_(_message.Message):
    __slots__ = ("algId", "algParams", "algData")
    ALGID_FIELD_NUMBER: _ClassVar[int]
    ALGPARAMS_FIELD_NUMBER: _ClassVar[int]
    ALGDATA_FIELD_NUMBER: _ClassVar[int]
    algId: str
    algParams: str
    algData: bytes
    def __init__(self, algId: _Optional[str] = ..., algParams: _Optional[str] = ..., algData: _Optional[bytes] = ...) -> None: ...

class Mission_(_message.Message):
    __slots__ = ("missionID", "shape", "algs")
    MISSIONID_FIELD_NUMBER: _ClassVar[int]
    SHAPE_FIELD_NUMBER: _ClassVar[int]
    ALGS_FIELD_NUMBER: _ClassVar[int]
    missionID: str
    shape: Shape_
    algs: _containers.RepeatedCompositeFieldContainer[AlgRequest_]
    def __init__(self, missionID: _Optional[str] = ..., shape: _Optional[_Union[Shape_, _Mapping]] = ..., algs: _Optional[_Iterable[_Union[AlgRequest_, _Mapping]]] = ...) -> None: ...

class AlgResponse_(_message.Message):
    __slots__ = ("algId", "result", "resp", "data")
    ALGID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    RESP_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    algId: str
    result: int
    resp: str
    data: bytes
    def __init__(self, algId: _Optional[str] = ..., result: _Optional[int] = ..., resp: _Optional[str] = ..., data: _Optional[bytes] = ...) -> None: ...

class MissionResponse_(_message.Message):
    __slots__ = ("missionId", "shape", "algs")
    MISSIONID_FIELD_NUMBER: _ClassVar[int]
    SHAPE_FIELD_NUMBER: _ClassVar[int]
    ALGS_FIELD_NUMBER: _ClassVar[int]
    missionId: str
    shape: Shape_
    algs: _containers.RepeatedCompositeFieldContainer[AlgResponse_]
    def __init__(self, missionId: _Optional[str] = ..., shape: _Optional[_Union[Shape_, _Mapping]] = ..., algs: _Optional[_Iterable[_Union[AlgResponse_, _Mapping]]] = ...) -> None: ...

class AlgorithmsRequest_(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class AlgorithmProp_(_message.Message):
    __slots__ = ("name", "fieldType", "defaultValue", "required", "label", "extValue")
    NAME_FIELD_NUMBER: _ClassVar[int]
    FIELDTYPE_FIELD_NUMBER: _ClassVar[int]
    DEFAULTVALUE_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_FIELD_NUMBER: _ClassVar[int]
    LABEL_FIELD_NUMBER: _ClassVar[int]
    EXTVALUE_FIELD_NUMBER: _ClassVar[int]
    name: str
    fieldType: str
    defaultValue: str
    required: bool
    label: str
    extValue: str
    def __init__(self, name: _Optional[str] = ..., fieldType: _Optional[str] = ..., defaultValue: _Optional[str] = ..., required: bool = ..., label: _Optional[str] = ..., extValue: _Optional[str] = ...) -> None: ...

class AlgorithmInfo_(_message.Message):
    __slots__ = ("algId", "algName", "props")
    ALGID_FIELD_NUMBER: _ClassVar[int]
    ALGNAME_FIELD_NUMBER: _ClassVar[int]
    PROPS_FIELD_NUMBER: _ClassVar[int]
    algId: str
    algName: str
    props: _containers.RepeatedCompositeFieldContainer[AlgorithmProp_]
    def __init__(self, algId: _Optional[str] = ..., algName: _Optional[str] = ..., props: _Optional[_Iterable[_Union[AlgorithmProp_, _Mapping]]] = ...) -> None: ...

class AlgorithmsResponse_(_message.Message):
    __slots__ = ("name", "algs")
    NAME_FIELD_NUMBER: _ClassVar[int]
    ALGS_FIELD_NUMBER: _ClassVar[int]
    name: str
    algs: _containers.RepeatedCompositeFieldContainer[AlgorithmInfo_]
    def __init__(self, name: _Optional[str] = ..., algs: _Optional[_Iterable[_Union[AlgorithmInfo_, _Mapping]]] = ...) -> None: ...

class DetectRequest_(_message.Message):
    __slots__ = ("name", "image", "missions", "exts")
    class ExtsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    IMAGE_FIELD_NUMBER: _ClassVar[int]
    MISSIONS_FIELD_NUMBER: _ClassVar[int]
    EXTS_FIELD_NUMBER: _ClassVar[int]
    name: str
    image: bytes
    missions: _containers.RepeatedCompositeFieldContainer[Mission_]
    exts: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., image: _Optional[bytes] = ..., missions: _Optional[_Iterable[_Union[Mission_, _Mapping]]] = ..., exts: _Optional[_Mapping[str, str]] = ...) -> None: ...

class DetectResponse_(_message.Message):
    __slots__ = ("name", "result", "message", "missions", "exts")
    class ExtsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MISSIONS_FIELD_NUMBER: _ClassVar[int]
    EXTS_FIELD_NUMBER: _ClassVar[int]
    name: str
    result: int
    message: str
    missions: _containers.RepeatedCompositeFieldContainer[MissionResponse_]
    exts: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., result: _Optional[int] = ..., message: _Optional[str] = ..., missions: _Optional[_Iterable[_Union[MissionResponse_, _Mapping]]] = ..., exts: _Optional[_Mapping[str, str]] = ...) -> None: ...

class PreprocessData_(_message.Message):
    __slots__ = ("algId", "result", "message", "algData")
    ALGID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ALGDATA_FIELD_NUMBER: _ClassVar[int]
    algId: str
    result: int
    message: str
    algData: bytes
    def __init__(self, algId: _Optional[str] = ..., result: _Optional[int] = ..., message: _Optional[str] = ..., algData: _Optional[bytes] = ...) -> None: ...

class MissionPreprocessResponse_(_message.Message):
    __slots__ = ("missionId", "algs")
    MISSIONID_FIELD_NUMBER: _ClassVar[int]
    ALGS_FIELD_NUMBER: _ClassVar[int]
    missionId: str
    algs: _containers.RepeatedCompositeFieldContainer[PreprocessData_]
    def __init__(self, missionId: _Optional[str] = ..., algs: _Optional[_Iterable[_Union[PreprocessData_, _Mapping]]] = ...) -> None: ...

class PreprocessResponse_(_message.Message):
    __slots__ = ("name", "result", "message", "missions", "exts")
    class ExtsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MISSIONS_FIELD_NUMBER: _ClassVar[int]
    EXTS_FIELD_NUMBER: _ClassVar[int]
    name: str
    result: int
    message: str
    missions: _containers.RepeatedCompositeFieldContainer[MissionPreprocessResponse_]
    exts: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., result: _Optional[int] = ..., message: _Optional[str] = ..., missions: _Optional[_Iterable[_Union[MissionPreprocessResponse_, _Mapping]]] = ..., exts: _Optional[_Mapping[str, str]] = ...) -> None: ...



from typing import List, Union, Dict

from google.protobuf.internal.containers import ScalarMap

from algservice.services.algservice.models import AlgorithmInfos, Algorithm, Mission, Rect, Point, AlgRequest, \
    PreprocessResponse, \
    DetectResponse, MissionResponse, AlgResponse, MissionPreprocessResponse, PreprocessData, AlgorithmProp
from .pb2utils import value_or_empty, bytes_or_empty

from algservice.util.exceptions import ErrorException, ErrorCode
from ..grpc.alg_service_pb2 import AlgorithmsResponse_, AlgorithmProp_, MissionPreprocessResponse_, PreprocessResponse_, \
    Shape_, MissionResponse_, DetectResponse_, AlgRequest_, Mission_, DetectRequest_, AlgorithmInfo_, Point_, \
    PreprocessData_, AlgResponse_


class pb2_to_local:
    """

    # 将 GRPC 数据转换为 Python Models

    """

    @staticmethod
    def to_AlgorithmInfos(s: AlgorithmsResponse_) -> AlgorithmInfos:
        x = AlgorithmInfos(s.name)
        for alg in s.algs:
            a = Algorithm(alg.algId, alg.algName)
            if alg.props is not None:
                for prop in alg.props:
                    pp = pb2_to_local.to_AlgorithmProp(prop)
                    a.alg_props.append(pp)

            x.add(a)

        return x

    @staticmethod
    def to_AlgorithmProp(prop: AlgorithmProp_) -> AlgorithmProp:
        out = AlgorithmProp(prop.name, prop.fieldType, prop.defaultValue, prop.label, prop.required, prop.extValue)
        return out

    ###################################

    @staticmethod
    def to_MissionPreprocessResponse(s: MissionPreprocessResponse_) -> MissionPreprocessResponse:
        x = MissionPreprocessResponse(s.missionId)
        for alg in s.algs:
            x.add(PreprocessData(alg.algId, alg.algData, alg.result, alg.message))
        return x

    @staticmethod
    def to_PreprocessResponse(s: PreprocessResponse_) -> PreprocessResponse:
        x = PreprocessResponse(s.name, s.result, s.message)
        for mission in s.missions:
            m = pb2_to_local.to_MissionPreprocessResponse(mission)
            x.add(m)

        x.exts = pb2_to_local.to_Dict_Str_Str(s.exts)
        return x

    @staticmethod
    def to_Dict_Str_Str(exts: ScalarMap[str, str]) -> Union[Dict[str, str], None]:

        size = len(exts)
        if size == 0:
            return None

        out: Dict[str, str] = dict()
        for key in exts.keys():
            val = exts.get(key)
            out[key] = val

        return out

    #################################

    @staticmethod
    def to_Shape(shape: Shape_) -> Union[Rect, List[Point]]:

        if shape.HasField("box"):
            b = Rect()
            b.top = shape.box.top
            b.left = shape.box.left
            b.width = shape.box.width
            b.height = shape.box.height
            return b

        if shape.HasField("polygon"):
            c: List[Point] = []
            for one in shape.polygon.points:
                c.append(Point(one.x, one.y))
            return c

        raise ErrorException(ErrorCode.invalid_argument, f"Unknown type - {type(shape)} , {shape.ListFields()}")

    @staticmethod
    def to_MissionResponse(s: MissionResponse_) -> MissionResponse:

        ns = pb2_to_local.to_Shape(s.shape)
        x = MissionResponse(s.missionId, ns)
        for alg in s.algs:
            x.add(AlgResponse(alg.algId, alg.result, alg.resp, alg.data))
        return x

    @staticmethod
    def to_DetectResponse(s: DetectResponse_) -> DetectResponse:
        x = DetectResponse(s.name, s.result, s.message)
        for mission in s.missions:
            m = pb2_to_local.to_MissionResponse(mission)
            x.add(m)
        x.exts = pb2_to_local.to_Dict_Str_Str(s.exts)
        return x

    #####################################################

    @staticmethod
    def to_AlgRequest(a: AlgRequest_) -> AlgRequest:
        b = AlgRequest()
        b.alg_id = a.algId
        b.alg_data = a.algData
        b.alg_params = a.algParams

        return b

    @staticmethod
    def to_Mission(m: Mission_) -> Mission:
        mission_id = m.missionID
        o = Mission(mission_id)
        o.shape = pb2_to_local.to_Shape(m.shape)
        for one in m.algs:
            alg = pb2_to_local.to_AlgRequest(one)
            o.add_alg(alg)
        return o

    @staticmethod
    def parse_DetectRequest(r: DetectRequest_) -> (str, bytes, List[Mission]):
        name = r.name
        image = r.image,
        missions: List[Mission] = []

        for mission in r.missions:
            nm = pb2_to_local.to_Mission(mission)
            missions.append(nm)

        return name, image, missions


#####################################################################################################################


class local_to_pb2:
    """

    # 将 Python Models  数据转换为 GRPC

    """

    @staticmethod
    def to_AlgorithmProp(prop: AlgorithmProp) -> AlgorithmProp_:
        out = AlgorithmProp_(name=value_or_empty(prop.name), fieldType=value_or_empty(prop.field_type),
                             defaultValue=value_or_empty(prop.default_value),
                             label=value_or_empty(prop.label), required=prop.required, extValue=prop.ext_value)
        return out

    @staticmethod
    def to_AlgorithmInfo(s: Algorithm) -> AlgorithmInfo_:
        x = AlgorithmInfo_(algId=value_or_empty(s.alg_id), algName=value_or_empty(s.alg_name))

        if s.alg_props is not None:
            for prop in s.alg_props:
                pp = local_to_pb2.to_AlgorithmProp(prop)
                x.props.append(pp)

        return x

    @staticmethod
    def to_AlgorithmsResponse(name: str, algs: List[Algorithm]) -> AlgorithmsResponse_:
        newalgs = [local_to_pb2.to_AlgorithmInfo(alg) for alg in algs]
        return AlgorithmsResponse_(name=value_or_empty(name), algs=newalgs)

    @staticmethod
    def to_ScalarMap_Str_Str(exts: Dict[str, str], out: ScalarMap[str, str]) -> ScalarMap[str, str]:

        if exts is None:
            return out
        for key in exts.keys():
            val = exts.get(key)
            out[key] = val

        return out

    @staticmethod
    def to_Shape_(shape) -> Shape_:

        if isinstance(shape, Rect):
            a = Shape_()
            b: Rect = shape
            a.box.top = b.top
            a.box.left = b.left
            a.box.width = b.width
            a.box.height = b.height
            return a

        if isinstance(shape, list):
            # should be List[Point]
            a = Shape_()
            b: List[Point] = shape
            for one in b:
                a.polygon.points.append(Point_(x=one.x, y=one.y))

        raise ErrorException(ErrorCode.invalid_argument, f"Unknown type - {type(shape)} , {shape}")

    @staticmethod
    def to_AlgRequest_(a: AlgRequest) -> AlgRequest_:

        c = AlgRequest_(algId=value_or_empty(a.alg_id), algData=bytes_or_empty(a.alg_data),
                        algParams=value_or_empty(a.alg_params))
        return c

    @staticmethod
    def to_Mission_(m: Mission) -> Mission_:
        shape = local_to_pb2.to_Shape_(m.shape)

        mission = Mission_(missionID=value_or_empty(m.mission_id), shape=shape)

        for one in m.algs:
            alg = local_to_pb2.to_AlgRequest_(one)
            mission.algs.append(alg)

        return mission

    @staticmethod
    def to_DetectRequest_(name: str, image: bytes, missions: List[Mission]) -> DetectRequest_:
        req = DetectRequest_(name=value_or_empty(name), image=bytes_or_empty(image))
        # local_to_pb2.to_ScalarMap_Str_Str(m.exts, req.exts)
        for mission in missions:
            nm = local_to_pb2.to_Mission_(mission)
            req.missions.append(nm)

        return req

    ##################################################

    @staticmethod
    def to_MissionPreprocessResponse_(m: MissionPreprocessResponse) -> MissionPreprocessResponse_:

        o = MissionPreprocessResponse_(missionId=value_or_empty(m.mission_id))
        for one in m.algs:
            o.algs.append(PreprocessData_(algId=value_or_empty(one.alg_id), algData=bytes_or_empty(one.alg_data)))
        return o

    @staticmethod
    def to_PreprocessResponse_(p: PreprocessResponse) -> PreprocessResponse_:
        req = PreprocessResponse_(name=value_or_empty(p.name), result=p.result, message=value_or_empty(p.message))
        for mission in p.missions:
            nm = local_to_pb2.to_MissionPreprocessResponse_(mission)
            req.missions.append(nm)
        local_to_pb2.to_ScalarMap_Str_Str(p.exts, req.exts)
        return req

    ##################################################

    @staticmethod
    def to_MissionResponse_(m: MissionResponse) -> MissionResponse_:
        shape = local_to_pb2.to_Shape_(m.shape)

        o = MissionResponse_(missionId=value_or_empty(m.mission_id),
                             shape=shape)

        for one in m.algs:
            o.algs.append(
                AlgResponse_(algId=value_or_empty(one.alg_id), result=one.result, resp=value_or_empty(one.resp),
                             data=bytes_or_empty(one.data)))
        return o

    @staticmethod
    def to_DetectResponse_(p: DetectResponse) -> DetectResponse_:
        req = DetectResponse_(name=value_or_empty(p.name), result=p.result, message=value_or_empty(p.message))
        for mission in p.missions:
            nm = local_to_pb2.to_MissionResponse_(mission)
            req.missions.append(nm)
        return req



from typing import List, Union, Dict




class AlgParam:

    def set_var(self, name, val):
        setattr(self, name, val);

    def has(self, name):
        return hasattr(self, name)

    def get(self, name):
        return getattr(self, name)

    def __init__(self, **kwargs):
        for name in kwargs:
            setattr(self, name, kwargs[name])

    def __setattr__(self, name, value):
        self.__dict__[name] = value

    def __getattr__(self, item):
        if item in self.__dict__:
            return self.__dict__[item]
        return None

    def __eq__(self, other):
        if not isinstance(other, AlgParam):
            return NotImplemented
        return vars(self) == vars(other)

    def __contains__(self, key):
        return key in self.__dict__


class AlgorithmProp:
    def __init__(self, name: str = "", field_type: str = "", default_value: str = "", label: str = "",
                 required: bool = False, ext_value:str=""):
        self.name = name
        self.field_type = field_type
        self.default_value = default_value
        self.label = label
        self.required = required
        self.ext_value = ext_value


class Algorithm:

    def __init__(self, alg_id: str, alg_name: str):
        self.alg_id = alg_id
        self.alg_name = alg_name
        self.alg_props: List[AlgorithmProp] = []

    def add(self, a: AlgorithmProp):
        self.alg_props.append(a)

    def add_props(self, a: List[AlgorithmProp]):
        if a is not None and len(a) > 0:
            self.alg_props.extend(a)


class AlgorithmInfos:
    def __init__(self, name):
        self.name = name
        self.algs: List[Algorithm] = []

    def add(self, one: Algorithm):
        self.algs.append(one)

    def get_alg_size(self):
        return len(self.algs)


class Rect:
    def __init__(self, left=0, top=0, width=0, height=0):
        self.left = left
        self.top = top
        self.width = width
        self.height = height

    def __str__(self):
        return f'left:{self.left} ,top:{self.top} ,width:{self.width} ,height:{self.height} '


class Point:
    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

    def __str__(self):
        return f'x:{self.x} ,y:{self.y}'


class AlgRequest:
    def __init__(self):
        self.alg_id = ""
        self.alg_params = ""
        self.alg_data: Union[bytes, None] = None

        self._alg_params_obj: Union[AlgParam, None] = None

    def set_params_obj(self, obj: AlgParam):
        self._alg_params_obj = obj

    def get_params_obj(self) -> Union[AlgParam, None]:
        """
        :return: 保存了，缓冲的  alg props 转换后的数值
        """
        return self._alg_params_obj


class Mission:

    def __init__(self, mission_id: str):
        self.mission_id = mission_id
        self.shape: Union[Rect, List[Point], None] = None
        self.algs: List[AlgRequest] = []

    def set_shape_rect(self, rect: Rect):
        self.shape = rect

    def set_shape_points(self, pts: List[Point]):
        self.shape = pts

    def add_alg(self, alg: AlgRequest):
        self.algs.append(alg)


class PreprocessData:
    def __init__(self, alg_id: str, alg_data: bytes, result: int, message: str):
        self.alg_id = alg_id
        self.alg_data = alg_data
        self.result = result
        self.message = message


class MissionPreprocessResponse:  # MissionPreprocessResponse_
    def __init__(self, mission_id: str):
        self.mission_id = mission_id
        self.algs: List[PreprocessData] = []

    def add(self, ad: PreprocessData):
        self.algs.append(ad)


class PreprocessResponse:  # PreprocessResponse_
    def __init__(self, name: str = "", result: int = 0, message: str = ""):
        self.name = name
        self.result = result
        self.message = message
        self.missions: List[MissionPreprocessResponse] = []
        self.exts: Union[Dict[str, str], None] = None

    def add(self, ad: MissionPreprocessResponse):
        self.missions.append(ad)


###################################################################################

class AlgResponse:
    def __init__(self, alg_id: str, result: int, resp: str, data: bytes):
        self.alg_id = alg_id
        self.result = result
        self.resp = resp
        self.data = data


class MissionResponse:
    def __init__(self, mission_id: str, shape: List[Point]):
        self.mission_id = mission_id

        self.shape = shape
        self.algs: List[AlgResponse] = []

    def add(self, ad: AlgResponse):
        self.algs.append(ad)


class DetectResponse:
    def __init__(self, name, result: int, message: str):
        self.name = name
        self.result = result
        self.message = message
        self.missions: List[MissionResponse] = []
        self.exts: Union[Dict[str, str], None] = None

    def add(self, ad: MissionResponse):
        self.missions.append(ad)



import random
import re
import shutil
import string
import tempfile
import time

import tarfile

import json
from datetime import datetime
import os
from pathlib import Path

 

from algservice.util.exceptions import ErrorException, ErrorCode


def is_array(arr):
    return isinstance(arr, list) or isinstance(arr, tuple)


# 将 yyyyMMddHHmmss 格式的字符串转换为 时间类型
def to_datetime(s: str) -> datetime:
    if is_blank(s):
        return None
    ln = len(s)
    if ln == 14:
        tm = datetime.strptime(s, '%Y%m%d%H%M%S')
    elif ln == 12:
        tm = datetime.strptime(s, '%Y%m%d%H%M')  # yyyyMMddHHmm
    elif ln == 10:
        tm = datetime.strptime(s, '%Y%m%d%H')  # yyyyMMddhh
    elif ln == 8:
        tm = datetime.strptime(s, '%Y%m%d')  # yyyyMMdd
    else:
        raise ErrorException(ErrorCode.invalid_argument, 'Unknown time {}'.format(s))

    return tm


def get_timestamp():
    nw = datetime.now()
    return nw.strftime('%Y_%m_%d_%H_%M_%S')


def is_num(s):
    if s is None:
        return False

    return isinstance(s, int)


def is_not_num(s):
    if s is None:
        return True

    return not isinstance(s, int)


def is_not_blank(s):
    if s is None:
        return False
    if isinstance(s, str):
        return bool(s and s.strip())
    return True


def is_blank(s):
    if s is None:
        return True

    if isinstance(s, str):
        if not bool(s):
            return True
        if not bool(s.strip()):
            return True

        return False
    return False


def convert_bytes(num):
    """
    this function will convert bytes to MB.... GB... etc
    """
    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0


def file_size(file_path):
    """
    this function will return the file size
    """
    if os.path.isfile(file_path):
        file_info = os.stat(file_path)
        return convert_bytes(file_info.st_size)


def get_dict_val(obj, name):
    if name in obj:
        return obj[name]
    return None


def get_extension2(filename):
    return Path(filename).suffix


def get_extension(file_path):
    # return Path(filename).
    return os.path.splitext(file_path)


def read_dict(obj, fullname, def_val=None):
    if obj is None:
        return def_val

    if is_blank(fullname):
        raise ErrorException(ErrorCode.invalid_argument, f"Invalid name : `{fullname}`")

    names = fullname.split('.')

    one = obj
    for name in names:
        if is_blank(name):
            raise ErrorException(ErrorCode.invalid_argument, f'Invalid name , `{name}` in `{fullname}`')

        if name in one:
            one = one[name]
        else:
            return def_val
    return one




def remove_file(filename, mute=True, throws=False):
    try:
        os.remove(filename)
    except Exception as e:
        if throws:
            raise e
        else:
            if not mute:
                print("remove file err ", filename, str(e));


def beans_to_arrays(beans, names):
    _out = {}
    for bean in beans:
        for name in names:
            val = read_dict(bean, name)
            if name not in _out:
                _out[name] = []

            array = _out[name]
            array.append(val)

    out = []
    for name in names:
        val = read_dict(_out, name)
        out.append(val)

    return out



def get_sys_tick():

    return int(time.time() * 1000)



def get_local_time_ms():
    return int(time.time() * 1000)


K1 = 1024
M1 = 1024 * 1024


def cal_buff_size(s):
    if s < K1:
        return "{}B".format(s)
    elif K1 <= s < M1:
        return "{:.1f}K".format(s / K1)
    else:
        return "{:.1f}M".format(s / M1)


def convert_bool(v: str) -> bool:
    if v is None or len(v) == 0:
        return False
    if len(v.strip()) == 0:
        return False

    sm = v.lower()
    if sm == "1" or sm == "true" or sm == "y" or sm == "yes":
        return True
    elif sm == "0" or sm == "false" or sm == "n" or sm == "no":
        return False
    else:
        raise ErrorException(ErrorCode.invalid_argument, "un wanted bool str: " + v)


class Struct(object):
    def __init__(self, data):
        for name, value in data.items():
            setattr(self, name, self._wrap(value))

    def _wrap(self, value):
        if isinstance(value, (tuple, list, set, frozenset)):
            return type(value)([self._wrap(v) for v in value])
        else:
            return Struct(value) if isinstance(value, dict) else value


def load_json_file_as_dict(fname):
    with open(fname, "r", encoding="utf-8") as f:
        return json.load(f)


def load_file(fname):
    with open(fname, "r", encoding="utf-8") as f:
        return f.read()

def load_binary_file(fname):
    with open(fname, "rb" ) as f:
        return f.read()


'''
Backup source_file to target directory `base_path` 
'''


def backup_file(source_file: str, base_path: str):
    config_base_path = os.path.dirname(source_file)
    basename = os.path.basename(source_file)
    filename, extension = os.path.splitext(basename)
    ts = get_timestamp()
    new_name = f'{filename}_{ts}{extension}'
    fullname = os.path.join(base_path, new_name)

    return shutil.copyfile(source_file, fullname)


def save_file(filename, buff):
    with open(filename, "wb") as f:
        f.write(buff)


def sanitize_filename(filename, replacement=''):
    """
     replace unsafe char in filename
    """
    # 使用正则表达式来匹配不安全的字符
    unsafe_chars = r'[/\?<>\\:*\|\"]'

    return re.sub(unsafe_chars, replacement, filename)


def get_file_list(parent):
    out = []
    # parent not exist
    if not os.path.exists(parent):
        return out

    for filename in os.listdir(parent):
        file = os.path.join(parent, filename)
        isdir = os.path.isdir(file)
        if not isdir:
            t = os.path.getmtime(file)
            tt = datetime.fromtimestamp(t)
            size = file_size(file)
            out.append({"name": filename, "type": "file", "size": size, "modified": tt})
    return out


def read_process_stdout(process, lines):
    stdout_list = []
    i = 0
    while i < lines:
        if process.poll():
            break

        line = process.stdout.readline()
        # Check if the line is empty, indicating that the subprocess has finished
        if len(line) > 0:
            stdout_list.append(str(line))
            i = i + 1
    return stdout_list


def read_process_stderr(process, lines):
    stdout_list = []
    i = 0
    while i < lines:
        if process.poll():
            break

        line = process.stderr.readline()
        # Check if the line is empty, indicating that the subprocess has finished
        # if not line:
        if len(line) > 0:
            stdout_list.append(str(line))
            i = i + 1
    return stdout_list


def is_simple_type(var):
    return isinstance(var, (int, float, str, bool))


def dump_list(vals, rep=0):
    prefix = ' ' * rep
    for value in vals:
        if is_simple_type(value):
            print("{} - {}".format(prefix, value))
        elif isinstance(value, (list, set)):
            print("{} - ".format(prefix, ))
        elif isinstance(value, (dict)):
            print("{} - ".format(prefix))
            dump_dict(value, rep + 2)
        else:
            print("{} - {}".format(prefix, value))


def dump_dict(val, rep=0):
    prefix = ' ' * rep
    for name, value in val.items():
        if is_simple_type(value):
            print("{}{} : {}".format(prefix, name, value))
        elif isinstance(value, (list, set)):
            dump_list(value, rep + 2)
        elif isinstance(value, (dict)):
            print("{}{} :".format(prefix, name))
            dump_dict(value, rep + 2)
        else:
            print("{}{} : {}".format(prefix, name, value))


# def load_yaml_config(filename):
#     with open(filename, "r", encoding="utf-8") as f:
#         data = yaml.safe_load(f)
#         return data


def compress_folder(folder_path, output_filename):
    # 创建.tar.gz压缩包
    with tarfile.open(output_filename, 'w:gz') as tar:
        for filename in os.listdir(folder_path):
            file_path = os.path.join(folder_path, filename)
            if os.path.isfile(file_path):
                tar.add(file_path, arcname=filename)


def get_tmp_dir():
    tempdir = tempfile.gettempdir()
    pathname = os.path.join(tempdir, 'backend-service')

    if not os.path.exists(pathname):
        os.makedirs(pathname)

    return pathname;


def get_user_home():
    user_home = os.environ.get('USERPROFILE', os.environ.get('HOME'))
    return user_home


tables = string.ascii_letters + string.digits


def get_rand_str(k=20) -> str:
    res = ''.join(random.choices(tables, k=k))
    return res


def sleep_ignore_exp(s: float):
    # try:
    #     time.sleep( s )
    # except KeyboardInterrupt:
    #     pass

    time.sleep(s)



#!/usr/bin/env python3
# _*_ coding: utf-8 _*_
__author__ = "L.K"

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("3rd.list", "r") as fh:
    requirements = fh.readlines()

pks = find_packages()  # .append('agent.py')
print(pks)

setup(name='alg-service-api-p12',
      version='1.0.5',  #
      description='alg service grpc api (for python3.12 )',  #
      author='L.K',  #
      author_email='L.K@idontwant.com',  #
      url='',  #
      packages=pks,  # Python导入包的列表，我们使用find_packages() 来自动收集
      long_description=long_description,  #
      long_description_content_type="text/markdown",  #

      classifiers=[
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.12",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Operating System :: POSIX :: Linux"
          ],

      python_requires='>=3.12',  # Python 的版本约束
      install_requires=requirements
      )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: algservice/services/grpc/alg-service.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*algservice/services/grpc/alg-service.proto\x12\x0cservice.grpc\x1a\x1bgoogle/protobuf/empty.proto\"\"\n\x0fServiceVersion_\x12\x0f\n\x07version\x18\x01 \x01(\t\"&\n\x05Size_\x12\r\n\x05width\x18\x01 \x01(\x02\x12\x0e\n\x06height\x18\x02 \x01(\x02\"\x1e\n\x06Point_\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\"A\n\x05Rect_\x12\x0c\n\x04left\x18\x01 \x01(\x02\x12\x0b\n\x03top\x18\x02 \x01(\x02\x12\r\n\x05width\x18\x03 \x01(\x02\x12\x0e\n\x06height\x18\x04 \x01(\x02\"0\n\x08Polygon_\x12$\n\x06points\x18\x01 \x03(\x0b\x32\x14.service.grpc.Point_\"_\n\x06Shape_\x12)\n\x07polygon\x18\x02 \x01(\x0b\x32\x16.service.grpc.Polygon_H\x00\x12\"\n\x03\x62ox\x18\x03 \x01(\x0b\x32\x13.service.grpc.Rect_H\x00\x42\x06\n\x04\x64\x61ta\"@\n\x0b\x41lgRequest_\x12\r\n\x05\x61lgId\x18\x01 \x01(\t\x12\x11\n\talgParams\x18\x02 \x01(\t\x12\x0f\n\x07\x61lgData\x18\x03 \x01(\x0c\"k\n\x08Mission_\x12\x11\n\tmissionID\x18\x01 \x01(\t\x12#\n\x05shape\x18\x02 \x01(\x0b\x32\x14.service.grpc.Shape_\x12\'\n\x04\x61lgs\x18\x03 \x03(\x0b\x32\x19.service.grpc.AlgRequest_\"I\n\x0c\x41lgResponse_\x12\r\n\x05\x61lgId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x0c\n\x04resp\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\"t\n\x10MissionResponse_\x12\x11\n\tmissionId\x18\x01 \x01(\t\x12#\n\x05shape\x18\x04 \x01(\x0b\x32\x14.service.grpc.Shape_\x12(\n\x04\x61lgs\x18\x05 \x03(\x0b\x32\x1a.service.grpc.AlgResponse_\"\"\n\x12\x41lgorithmsRequest_\x12\x0c\n\x04name\x18\x01 \x01(\t\"z\n\x0e\x41lgorithmProp_\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfieldType\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x65\x66\x61ultValue\x18\x03 \x01(\t\x12\x10\n\x08required\x18\x05 \x01(\x08\x12\r\n\x05label\x18\x06 \x01(\t\x12\x10\n\x08\x65xtValue\x18\x07 \x01(\t\"]\n\x0e\x41lgorithmInfo_\x12\r\n\x05\x61lgId\x18\x01 \x01(\t\x12\x0f\n\x07\x61lgName\x18\x02 \x01(\t\x12+\n\x05props\x18\x03 \x03(\x0b\x32\x1c.service.grpc.AlgorithmProp_\"O\n\x13\x41lgorithmsResponse_\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\x04\x61lgs\x18\x02 \x03(\x0b\x32\x1c.service.grpc.AlgorithmInfo_\"\xba\x01\n\x0e\x44\x65tectRequest_\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05image\x18\x02 \x01(\x0c\x12(\n\x08missions\x18\x03 \x03(\x0b\x32\x16.service.grpc.Mission_\x12\x34\n\x04\x65xts\x18\x04 \x03(\x0b\x32&.service.grpc.DetectRequest_.ExtsEntry\x1a+\n\tExtsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd6\x01\n\x0f\x44\x65tectResponse_\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x30\n\x08missions\x18\x04 \x03(\x0b\x32\x1e.service.grpc.MissionResponse_\x12\x35\n\x04\x65xts\x18\x05 \x03(\x0b\x32\'.service.grpc.DetectResponse_.ExtsEntry\x1a+\n\tExtsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"R\n\x0fPreprocessData_\x12\r\n\x05\x61lgId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x0f\n\x07\x61lgData\x18\x04 \x01(\x0c\"\\\n\x1aMissionPreprocessResponse_\x12\x11\n\tmissionId\x18\x01 \x01(\t\x12+\n\x04\x61lgs\x18\x02 \x03(\x0b\x32\x1d.service.grpc.PreprocessData_\"\xe8\x01\n\x13PreprocessResponse_\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\x12:\n\x08missions\x18\x04 \x03(\x0b\x32(.service.grpc.MissionPreprocessResponse_\x12\x39\n\x04\x65xts\x18\x05 \x03(\x0b\x32+.service.grpc.PreprocessResponse_.ExtsEntry\x1a+\n\tExtsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\xc5\x02\n\nAlgService\x12\x45\n\nGetVersion\x12\x16.google.protobuf.Empty\x1a\x1d.service.grpc.ServiceVersion_\"\x00\x12V\n\rGetAlgorithms\x12 .service.grpc.AlgorithmsRequest_\x1a!.service.grpc.AlgorithmsResponse_\"\x00\x12O\n\nPreprocess\x12\x1c.service.grpc.DetectRequest_\x1a!.service.grpc.PreprocessResponse_\"\x00\x12G\n\x06\x44\x65tect\x12\x1c.service.grpc.DetectRequest_\x1a\x1d.service.grpc.DetectResponse_\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'algservice.services.grpc.alg_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DETECTREQUEST__EXTSENTRY']._loaded_options = None
  _globals['_DETECTREQUEST__EXTSENTRY']._serialized_options = b'8\001'
  _globals['_DETECTRESPONSE__EXTSENTRY']._loaded_options = None
  _globals['_DETECTRESPONSE__EXTSENTRY']._serialized_options = b'8\001'
  _globals['_PREPROCESSRESPONSE__EXTSENTRY']._loaded_options = None
  _globals['_PREPROCESSRESPONSE__EXTSENTRY']._serialized_options = b'8\001'
  _globals['_SERVICEVERSION_']._serialized_start=89
  _globals['_SERVICEVERSION_']._serialized_end=123
  _globals['_SIZE_']._serialized_start=125
  _globals['_SIZE_']._serialized_end=163
  _globals['_POINT_']._serialized_start=165
  _globals['_POINT_']._serialized_end=195
  _globals['_RECT_']._serialized_start=197
  _globals['_RECT_']._serialized_end=262
  _globals['_POLYGON_']._serialized_start=264
  _globals['_POLYGON_']._serialized_end=312
  _globals['_SHAPE_']._serialized_start=314
  _globals['_SHAPE_']._serialized_end=409
  _globals['_ALGREQUEST_']._serialized_start=411
  _globals['_ALGREQUEST_']._serialized_end=475
  _globals['_MISSION_']._serialized_start=477
  _globals['_MISSION_']._serialized_end=584
  _globals['_ALGRESPONSE_']._serialized_start=586
  _globals['_ALGRESPONSE_']._serialized_end=659
  _globals['_MISSIONRESPONSE_']._serialized_start=661
  _globals['_MISSIONRESPONSE_']._serialized_end=777
  _globals['_ALGORITHMSREQUEST_']._serialized_start=779
  _globals['_ALGORITHMSREQUEST_']._serialized_end=813
  _globals['_ALGORITHMPROP_']._serialized_start=815
  _globals['_ALGORITHMPROP_']._serialized_end=937
  _globals['_ALGORITHMINFO_']._serialized_start=939
  _globals['_ALGORITHMINFO_']._serialized_end=1032
  _globals['_ALGORITHMSRESPONSE_']._serialized_start=1034
  _globals['_ALGORITHMSRESPONSE_']._serialized_end=1113
  _globals['_DETECTREQUEST_']._serialized_start=1116
  _globals['_DETECTREQUEST_']._serialized_end=1302
  _globals['_DETECTREQUEST__EXTSENTRY']._serialized_start=1259
  _globals['_DETECTREQUEST__EXTSENTRY']._serialized_end=1302
  _globals['_DETECTRESPONSE_']._serialized_start=1305
  _globals['_DETECTRESPONSE_']._serialized_end=1519
  _globals['_DETECTRESPONSE__EXTSENTRY']._serialized_start=1259
  _globals['_DETECTRESPONSE__EXTSENTRY']._serialized_end=1302
  _globals['_PREPROCESSDATA_']._serialized_start=1521
  _globals['_PREPROCESSDATA_']._serialized_end=1603
  _globals['_MISSIONPREPROCESSRESPONSE_']._serialized_start=1605
  _globals['_MISSIONPREPROCESSRESPONSE_']._serialized_end=1697
  _globals['_PREPROCESSRESPONSE_']._serialized_start=1700
  _globals['_PREPROCESSRESPONSE_']._serialized_end=1932
  _globals['_PREPROCESSRESPONSE__EXTSENTRY']._serialized_start=1259
  _globals['_PREPROCESSRESPONSE__EXTSENTRY']._serialized_end=1302
  _globals['_ALGSERVICE']._serialized_start=1935
  _globals['_ALGSERVICE']._serialized_end=2260
# @@protoc_insertion_point(module_scope)



import base64
import datetime
import json
import threading
from typing import List

from algservice.util.syncutils import locked
from algservice.util.utils import to_datetime, is_not_blank


class ObjectJsonEncoder:
    def get_class(self):
        """
        interface virtual function
        :return:
        """
        pass

    def match(self, clazz) -> bool:
        """
        interface virtual function
        :return:
        """
        return False

    def convert(self, obj):
        return None


class DefaultEncoder(ObjectJsonEncoder):


    def match(self, clazz) -> bool:
        return issubclass(clazz, (list, set))

    def convert(self, obj):
        if isinstance(obj, list):
            return obj
        if isinstance(obj, set):
            return list(obj)
        return None


class DatetimeEncoder(ObjectJsonEncoder):

    def match(self, clazz) -> bool:
        return issubclass(clazz, datetime.datetime)

    def convert(self, obj) -> dict:
        return obj.strftime('%Y-%m-%d %H:%M:%S.%f')


_datetimeEncoder = DatetimeEncoder()
_defaultEncoder = DefaultEncoder()


class EncoderCache:
    def __init__(self):
        self._cache = {}
        self._lock = threading.Lock()

    @locked(lock="_lock")
    def get_encoder(self, clazz):
        if clazz in self._cache:
            return self._cache[clazz]
        return None

    @locked(lock="_lock")
    def set_encoder(self, clazz, encoder):
        self._cache[clazz] = encoder


class ComplexEncoderHelper:

    def __init__(self):
        self._cache = EncoderCache()
        self.registers: List[ObjectJsonEncoder] = []

    def register(self, encoder: ObjectJsonEncoder):
        self.registers.append(encoder)

    def _find_encoder(self, clazz):
        for r in self.registers:
            if r.match(clazz):
                return r

        return None

    def find_encoder(self, clazz) -> ObjectJsonEncoder:
        global _datetimeEncoder
        global _defaultEncoder

        if _defaultEncoder.match(clazz):
            return _defaultEncoder
        if _datetimeEncoder.match(clazz):
            return _datetimeEncoder

        return self._find_encoder(clazz)

    def convert(self, obj) -> any:
        clazz = obj.__class__
        encoder = self._cache.get_encoder(clazz)
        if encoder is None:
            encoder = self.find_encoder(clazz)
            if encoder is not None:
                self._cache.set_encoder(clazz, encoder)
            else:
                return None

        return encoder.convert(obj)


_complexEncoderHelper = ComplexEncoderHelper()


def register_encoder(encoder: ObjectJsonEncoder):
    global _complexEncoderHelper
    _complexEncoderHelper.register(encoder)


#
class ComplexEncoderEx(json.JSONEncoder):
    def default(self, obj):
        # print(" in default , ", obj.__class__, obj);
        global _complexEncoderHelper
        val = _complexEncoderHelper.convert(obj)
        if val is not None:
            return val
        else:
            if obj.__dict__ is not None:
                return obj.__dict__
            else:
                return super().default(obj)


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):

        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S.%f')
        elif isinstance(obj, set):
            return list(obj)
        elif isinstance(obj, list):
            return obj
        elif isinstance(obj, bytes):
            # return f'base64:{base64.b64encode(obj).decode("utf-8")}'
            return base64.b64encode(obj).decode("utf-8")

        elif isinstance(obj, object):
            return obj.__dict__
        else:
            return super().default(obj)



def to_json_str2(obj):
    s = json.dumps(obj, cls=ComplexEncoderEx)
    return s


def to_json_str(obj):
    s = json.dumps(obj, cls=ComplexEncoder)
    return s


def json_to_dict(buff):
    return json.loads(buff)



def read_dict_val (obj, name):
    if name in obj:
        return obj[name]
    return None

def read_as_bytes (obj, name, encode="base64"):

    val = read_dict_val(obj, name)
    if val is None:
        return None

    if isinstance(val,bytes )  :
        return val

    if isinstance(val,str ):
        if encode == "base64":
            if val.startswith("base64:") :
                val = val[7:]
            return base64.b64decode( val )



    return val


def read_datetime(obj, name):
    start = read_dict_val(obj, name)
    if start is not None and len(start) > 0:
        return to_datetime(start)

    return None


def read_int(obj, name):
    value = read_dict_val(obj, name)
    if is_not_blank(value):
        return int(value)

    return None





# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from algservice.services.grpc import alg_service_pb2 as algservice_dot_services_dot_grpc_dot_alg__service__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

GRPC_GENERATED_VERSION = '1.64.1'
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = '1.65.0'
SCHEDULED_RELEASE_DATE = 'June 25, 2024'
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in algservice/services/grpc/alg_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
        + f' This warning will become an error in {EXPECTED_ERROR_RELEASE},'
        + f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',
        RuntimeWarning
    )


class AlgServiceStub(object):
    """Interface exported by the server.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetVersion = channel.unary_unary(
                '/service.grpc.AlgService/GetVersion',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.ServiceVersion_.FromString,
                _registered_method=True)
        self.GetAlgorithms = channel.unary_unary(
                '/service.grpc.AlgService/GetAlgorithms',
                request_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsRequest_.SerializeToString,
                response_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsResponse_.FromString,
                _registered_method=True)
        self.Preprocess = channel.unary_unary(
                '/service.grpc.AlgService/Preprocess',
                request_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.SerializeToString,
                response_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.PreprocessResponse_.FromString,
                _registered_method=True)
        self.Detect = channel.unary_unary(
                '/service.grpc.AlgService/Detect',
                request_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.SerializeToString,
                response_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectResponse_.FromString,
                _registered_method=True)


class AlgServiceServicer(object):
    """Interface exported by the server.
    """

    def GetVersion(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAlgorithms(self, request, context):
        """返回 当前支持的所有算法
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Preprocess(self, request, context):
        """预处理，在注册阶段， 背景图+ 图形+算法+参数 需要获取一段兹定于数据
        ： 输入 注册图像，检测任务列表，算法参数， 返回 预处理数据，这个数据应该调用者保存，将来在检测时
        使用  
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Detect(self, request, context):
        """检测： 输入 实时图像，检测任务列表， 返回 检测结果列表 

        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AlgServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVersion,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.ServiceVersion_.SerializeToString,
            ),
            'GetAlgorithms': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAlgorithms,
                    request_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsRequest_.FromString,
                    response_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsResponse_.SerializeToString,
            ),
            'Preprocess': grpc.unary_unary_rpc_method_handler(
                    servicer.Preprocess,
                    request_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.FromString,
                    response_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.PreprocessResponse_.SerializeToString,
            ),
            'Detect': grpc.unary_unary_rpc_method_handler(
                    servicer.Detect,
                    request_deserializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.FromString,
                    response_serializer=algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectResponse_.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'service.grpc.AlgService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('service.grpc.AlgService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class AlgService(object):
    """Interface exported by the server.
    """

    @staticmethod
    def GetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/service.grpc.AlgService/GetVersion',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.ServiceVersion_.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAlgorithms(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/service.grpc.AlgService/GetAlgorithms',
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsRequest_.SerializeToString,
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.AlgorithmsResponse_.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Preprocess(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/service.grpc.AlgService/Preprocess',
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.SerializeToString,
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.PreprocessResponse_.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Detect(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/service.grpc.AlgService/Detect',
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectRequest_.SerializeToString,
            algservice_dot_services_dot_grpc_dot_alg__service__pb2.DetectResponse_.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)
